################################################################################
# File:             figure4.R
# Created by:       Pierre Lafaye De Micheaux, Pavlo Mozharovskyi
# First released:   27.10.2018
# Last revised:     27.10.2018
# 
# Contains R-script for construction of Figure 4:
#   - Whole brain fiber dataset (Figure 4, left).
#   - Result of bundle ordering for the right side of the brain 
#     only (Figure 4, right).
# 
# For a detailed description see:
#   Lafaye De Micheaux, P., Mozharovskyi, P. and Vimond, M. (2018). Depth for 
#     curve data and applications.
################################################################################

# Load R libraries and a few other functions to read DTI data
require("curveDepth")
require("misc3d")
require("rgl")
require("AnalyzeFMRI")
source("DTI.R")

# Load the anatomical brain volume data
X <- f.read.volume("101_1_dwi_fa.nii")
X.hdr <- f.read.nifti.header("101_1_dwi_fa.nii")
X.hdr$dim
X <- X[, , , 1] # Since there is no time involved in DTI data
dim.X <- dim(X)

# Load the DTI fibers data
fibers.Left <- f.read.fibers(fibers.file = "101_1_dwi.voxelcoordsL.txt")
fibers.Left.coord <- fibers.Left$fibers.coordinates
fibers.Right <- f.read.fibers(fibers.file = "101_1_dwi.voxelcoordsR.txt")
fibers.Right.coord <- fibers.Right$fibers.coordinates

# This data set contains two bundles (left and right) of 1,000 fibers each
nrow(fibers.Left$indices.of.fibers)
nrow(fibers.Right$indices.of.fibers)
# but we will only display the first 100 fibers
n <- 100
leftFibers <- list("")
rightFibers <- list("")

# Extract 100 first left fibers
for (j in 1:n) {
  f1 <- fibers.Left$indices.of.fibers[j]
  f2 <- fibers.Left$indices.of.fibers[j + 1] - 1
  x <- fibers.Left.coord[f1:f2, 1]
  y <- fibers.Left.coord[f1:f2, 2]
  z <- fibers.Left.coord[f1:f2, 3]
  leftFibers[[j]] <- list(voxels = 0, coords = cbind(x, y, z))
}

# Extract 100 first right fibers
for (j in 1:n) {
  f1 <- fibers.Right$indices.of.fibers[j]
  f2 <- fibers.Right$indices.of.fibers[j + 1] - 1
  x <- fibers.Right.coord[f1:f2, 1]
  y <- fibers.Right.coord[f1:f2, 2]
  z <- fibers.Right.coord[f1:f2, 3]
  rightFibers[[j]] <- list(voxels = 0, coords = cbind(x, y, z))
}

# Calculate depths
set.seed(1)
# The calculation can take quite some time (from 0.5 to 1 hour, 
# for both hemispheres, depending on the computer speed)
dssLeft <- depthc.Tukey(leftFibers, leftFibers, 
                        m = 250L, nDirs = 1000, exactEst = FALSE)
dssRight <- depthc.Tukey(rightFibers, rightFibers, 
                         m = 250L, nDirs = 1000, exactEst = FALSE)
# Save the results to for the web applet
save.image("figure4.RData")

#### Figure 4 (Left)

# Create depth-based color palettes an determine color of each curve
leftNColors <- length(dssLeft) + 1
my_palette <- colorRampPalette(c("yellow", "red"))(n = leftNColors)
dssLeft.colors <- my_palette[floor((dssLeft - min(dssLeft)) /
                                     (max(dssLeft) - min(dssLeft)) * 
                                     (leftNColors - 1)) + 1]
dssLeft.colors[which.max(dssLeft)] <- "blue"
rightNColors <- length(dssRight) + 1
my_palette <- colorRampPalette(c("yellow", "red"))(n = rightNColors)
dssRight.colors <- my_palette[floor((dssRight - min(dssRight)) /
                                      (max(dssRight) - min(dssRight)) * 
                                      (rightNColors - 1)) + 1]
dssRight.colors[which.max(dssRight)] <- "blue"

# Display the brain volum using rgl
contour3d(X, x = 1:dim.X[1], y = 1:dim.X[2], z = 1:dim.X[3], lev = 0.4, 
          alpha = 0.2, color = "grey", add = TRUE, fill = FALSE)

# Plot left fibers
for (j in 1:n) {
  lines3d(leftFibers[[j]]$coords[, 1], 
          leftFibers[[j]]$coords[, 2], 
          leftFibers[[j]]$coords[, 3], 
          col = dssLeft.colors[j], add = TRUE, lwd = 3)
}

# Plot right fibers
for (j in 1:n) {
  lines3d(rightFibers[[j]]$coords[, 1], 
          rightFibers[[j]]$coords[, 2], 
          rightFibers[[j]]$coords[, 3], 
          col = dssRight.colors[j], add = TRUE, lwd = 3)
}

# Change the orientation and viewpoint of the scene
rgl.viewpoint(theta = -25, phi = -45, fov = 30, zoom = 0.7)

#### Figure 4 (Right)

# Split/trim the curves of the right part of the brain according to their depth
barplot(sort(dssRight, decreasing = TRUE))
abline(h = 0.075) # Not shown in the paper
iOrder <- order(dssRight, decreasing = TRUE)
iMostCentral <- which.max(dssRight)
iCentral <- order(dssRight, decreasing = TRUE)[1:floor(n / 2)]
iCentral <- iCentral[-1]
iOutsider <- which(dssRight < 0.075)
iOuter <- (1:length(dssRight))[-c(iMostCentral, iCentral, iOutsider)]

# Plot the box-plot for the right part of the brain
rgl.open()
rgl.bg(color = "white")
col1 <- c("royalblue", "red", "royalblue4")

# Plot outer fibers
for (i in iOuter){
  lines3d(rightFibers[[i]]$coords[, 1], 
          rightFibers[[i]]$coords[, 2], 
          rightFibers[[i]]$coords[, 3], 
          col = col1[1], add =TRUE, lwd = 3, alpha = 0.25)
}

# Plot central fibers
for (i in iCentral){
  lines3d(rightFibers[[i]]$coords[, 1], 
          rightFibers[[i]]$coords[, 2], 
          rightFibers[[i]]$coords[, 3], 
          col = col1[1], add = TRUE, lwd = 3, alpha = 0.75)
}

# Plot outliers
for (i in iOutsider){
  lines3d(rightFibers[[i]]$coords[, 1], 
          rightFibers[[i]]$coords[, 2], 
          rightFibers[[i]]$coords[, 3], 
          col = col1[2], add = TRUE, lwd = 3, alpha = 0.5)
}

# Plot the deepest curve
lines3d(rightFibers[[iMostCentral]]$coords[, 1], 
        rightFibers[[iMostCentral]]$coords[, 2], 
        rightFibers[[iMostCentral]]$coords[, 3], 
        col = col1[3], add = TRUE, lwd = 3, alpha = 1)

# Change the orientation and viewpoint of the scene
rgl.viewpoint(theta = -25, phi = -45, fov = 30, zoom = 0.6)
